"use strict";

var __awaiter = undefined && undefined.__awaiter || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) {
            try {
                step(generator.next(value));
            } catch (e) {
                reject(e);
            }
        }
        function rejected(value) {
            try {
                step(generator["throw"](value));
            } catch (e) {
                reject(e);
            }
        }
        function step(result) {
            result.done ? resolve(result.value) : new P(function (resolve) {
                resolve(result.value);
            }).then(fulfilled, rejected);
        }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const mkdirp_1 = require("mkdirp");
const path_1 = require("path");
const Generator = require("yeoman-generator");
const pkg = require('../../package.json');
const versions = {};
Object.keys(pkg.dependencies).forEach(name => {
    versions['version_' + name.replace(/[^a-zA-Z]/g, '_')] = pkg.dependencies[name];
});
Object.keys(pkg.devDependencies).forEach(name => {
    versions['version_' + name.replace(/[^a-zA-Z]/g, '_')] = pkg.devDependencies[name];
});
module.exports = class MopedGenerator extends Generator {
    constructor(args, options) {
        super(args, options);
        this.props = Object.assign({}, versions);
        this.options = options;
        this.argument('name', { type: String, required: false });
    }
    prompting() {
        return __awaiter(this, void 0, void 0, function* () {
            // Have Yeoman greet the user.
            this.log('Welcome to ' + chalk_1.default.red('moped') + '!');
            if (this.options.name) {
                this.props.name = this.options.name;
            } else {
                const { name } = yield this.prompt([{
                    type: 'input',
                    name: 'name',
                    message: 'Your app name (e.g. example.com)',
                    default: path_1.basename(process.cwd())
                }]);
                this.props.name = name;
            }
            this.props.dbname = this.props.name.replace(/[^a-z0-9_\-]/gi, '_');
        });
    }
    directory() {
        const name = this.props.name;
        if (!name) {
            throw new Error('You must supply a name');
        }
        if (path_1.basename(this.destinationPath()) !== name) {
            this.log('Your app must be inside a folder named ' + name + '\n' + "I'll automatically create this folder.");
            const dir = this.destinationPath(name);
            mkdirp_1.sync(dir);
            this.destinationRoot(dir);
            process.chdir(dir);
        }
    }
    writeOutput() {
        this.fs.copyTpl(this.templatePath('env'), this.destinationPath('.env'), this.props);
        this.fs.copy(this.templatePath('gitignore'), this.destinationPath('.gitignore'));
        this.fs.copy(this.templatePath('prettierrc'), this.destinationPath('.prettierrc'));
        this.fs.copyTpl(this.templatePath('package.json'), this.destinationPath('package.json'), this.props);
        this.fs.copy(this.templatePath('tsconfig.json'), this.destinationPath('tsconfig.json'));
        mkdirp_1.sync(this.destinationPath('src'));
        this.fs.copy(this.templatePath('src/client.tsx'), this.destinationPath('src/client.tsx'));
        this.fs.copy(this.templatePath('src/email.ts'), this.destinationPath('src/email.ts'));
        this.fs.copyTpl(this.templatePath('src/index.html'), this.destinationPath('src/index.html'), this.props);
        this.fs.copy(this.templatePath('src/server.dev.ts'), this.destinationPath('src/server.dev.ts'));
        this.fs.copy(this.templatePath('src/server.prod.ts'), this.destinationPath('src/server.prod.ts'));
        this.fs.copy(this.templatePath('src/server.tsx'), this.destinationPath('src/server.tsx'));
        mkdirp_1.sync(this.destinationPath('src/authentication'));
        this.fs.copy(this.templatePath('src/authentication/passwordless.ts'), this.destinationPath('src/authentication/passwordless.ts'));
        mkdirp_1.sync(this.destinationPath('src/bicycle-schema'));
        this.fs.copy(this.templatePath('src/bicycle-schema/BicycleContext.ts'), this.destinationPath('src/bicycle-schema/BicycleContext.ts'));
        this.fs.copy(this.templatePath('src/bicycle-schema/Root.ts'), this.destinationPath('src/bicycle-schema/Root.ts'));
        this.fs.copy(this.templatePath('src/bicycle-schema/User.ts'), this.destinationPath('src/bicycle-schema/User.ts'));
        mkdirp_1.sync(this.destinationPath('src/components'));
        this.fs.copy(this.templatePath('src/components/App.tsx'), this.destinationPath('src/components/App.tsx'));
        this.fs.copy(this.templatePath('src/components/ErrorBoundary.tsx'), this.destinationPath('src/components/ErrorBoundary.tsx'));
        this.fs.copy(this.templatePath('src/components/Home.tsx'), this.destinationPath('src/components/Home.tsx'));
        this.fs.copy(this.templatePath('src/components/LoginPage.tsx'), this.destinationPath('src/components/LoginPage.tsx'));
        this.fs.copy(this.templatePath('src/components/PageNotFound.tsx'), this.destinationPath('src/components/PageNotFound.tsx'));
        this.fs.copy(this.templatePath('src/components/ProfilePage.tsx'), this.destinationPath('src/components/ProfilePage.tsx'));
        mkdirp_1.sync(this.destinationPath('src/db-migrations'));
        this.fs.copy(this.templatePath('src/db-migrations/00001-init.ts'), this.destinationPath('src/db-migrations/00001-init.ts'));
        this.fs.copy(this.templatePath('src/db-connection.ts'), this.destinationPath('src/db-connection.ts'));
        this.fs.copy(this.templatePath('src/db.ts'), this.destinationPath('src/db.ts'));
        mkdirp_1.sync(this.destinationPath('src/public'));
    }
    install() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.yarnInstall();
        });
    }
};
//# sourceMappingURL=index.js.map