"use strict";

var __awaiter = undefined && undefined.__awaiter || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) {
            try {
                step(generator.next(value));
            } catch (e) {
                reject(e);
            }
        }
        function rejected(value) {
            try {
                step(generator["throw"](value));
            } catch (e) {
                reject(e);
            }
        }
        function step(result) {
            result.done ? resolve(result.value) : new P(function (resolve) {
                resolve(result.value);
            }).then(fulfilled, rejected);
        }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const mkdirp_1 = require("mkdirp");
const path_1 = require("path");
const Generator = require("yeoman-generator");
const pkg = require('../../package.json');
const versions = {};
Object.keys(pkg.dependencies).forEach(name => {
    versions['version_' + name.replace(/[^a-zA-Z]/g, '_')] = pkg.dependencies[name];
});
Object.keys(pkg.devDependencies).forEach(name => {
    versions['version_' + name.replace(/[^a-zA-Z]/g, '_')] = pkg.devDependencies[name];
});
module.exports = class MopedGenerator extends Generator {
    constructor(args, options) {
        super(args, options);
        this.props = Object.assign({}, versions, { year: '' + new Date().getFullYear() });
        this.options = options;
        this.argument('name', { type: String, required: false });
    }
    prompting() {
        return __awaiter(this, void 0, void 0, function* () {
            // Have Yeoman greet the user.
            this.log('Welcome to ' + chalk_1.default.red('moped:library') + '!');
            if (this.options.name) {
                this.props.name = this.options.name;
            } else {
                const { name } = yield this.prompt([{
                    type: 'input',
                    name: 'name',
                    message: 'Your library name',
                    default: path_1.basename(process.cwd())
                }]);
                this.props.name = name;
            }
            if (this.options.ownerName) {
                this.props.ownerName = this.options.ownerName;
            } else {
                const { ownerName } = yield this.prompt([{
                    type: 'input',
                    name: 'ownerName',
                    message: 'Your GitHub user name',
                    default: 'ForbesLindesay'
                }]);
                this.props.ownerName = ownerName;
            }
        });
    }
    directory() {
        const name = this.props.name;
        if (!name) {
            throw new Error('You must supply a name');
        }
        if (path_1.basename(this.destinationPath()) !== name) {
            this.log('Your app must be inside a folder named ' + name + '\n' + "I'll automatically create this folder.");
            const dir = this.destinationPath(name);
            mkdirp_1.sync(dir);
            this.destinationRoot(dir);
            process.chdir(dir);
        }
    }
    writeOutput() {
        this.fs.copy(this.templatePath('prettierrc'), this.destinationPath('.prettierrc'));
        this.fs.copy(this.templatePath('travis.yml'), this.destinationPath('.travis.yml'));
        this.fs.copy(this.templatePath('gitignore'), this.destinationPath('.gitignore'));
        this.fs.copy(this.templatePath('LICENSE'), this.destinationPath('LICENSE'));
        this.fs.copyTpl(this.templatePath('package.json'), this.destinationPath('package.json'), this.props);
        this.fs.copy(this.templatePath('tsconfig.json'), this.destinationPath('tsconfig.json'));
        mkdirp_1.sync(this.destinationPath('src'));
        this.fs.copy(this.templatePath('src/index.ts'), this.destinationPath('src/index.ts'));
        mkdirp_1.sync(this.destinationPath('src/__tests__'));
        this.fs.copy(this.templatePath('src/__tests__/index.test.ts'), this.destinationPath('src/__tests__/index.test.ts'));
    }
    install() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.yarnInstall();
        });
    }
};
//# sourceMappingURL=index.js.map